@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2010 IBM Corporation and others.
+ * Copyright (c) 2000, 2011 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -335,6 +335,16 @@ int TranslateAccelerator(int /*long*/ lpMsg, int /*long*/ pguidCmdGroup, int nCm
 				 * if it will be within IE or out to another Control.
 				 */
 				break;
+			case OS.VK_UP:
+			case OS.VK_DOWN:
+			case OS.VK_LEFT:
+			case OS.VK_RIGHT:
+			case OS.VK_HOME:
+			case OS.VK_END:
+			case OS.VK_PRIOR:
+			case OS.VK_NEXT:
+				/* Do not translate/consume IE's keys for scrolling content. */
+				break;
 			case OS.VK_BACK:
 			case OS.VK_RETURN:
 				/*
@@ -712,14 +722,14 @@ int Invoke (int dispIdMember, int /*long*/ riid, int lcid, int dwFlags, int /*lo
 
 	DISPPARAMS dispParams = new DISPPARAMS ();
 	COM.MoveMemory (dispParams, pDispParams, DISPPARAMS.sizeof);
-	if (dispParams.cArgs != 2) {
+	if (dispParams.cArgs != 3) {
 		if (pVarResult != 0) {
 			COM.MoveMemory (pVarResult, new int /*long*/[] {0}, C.PTR_SIZEOF);
 		}
 		return COM.S_OK;
 	}
 
-	int /*long*/ ptr = dispParams.rgvarg + Variant.sizeof;
+	int /*long*/ ptr = dispParams.rgvarg + 2 * Variant.sizeof;
 	Variant variant = Variant.win32_new (ptr);
 	if (variant.getType () != COM.VT_I4) {
 		variant.dispose ();
@@ -737,11 +747,24 @@ int Invoke (int dispIdMember, int /*long*/ riid, int lcid, int dwFlags, int /*lo
 		return COM.S_OK;
 	}
 
+	ptr = dispParams.rgvarg + Variant.sizeof;
+	variant = Variant.win32_new (ptr);
+	int type = variant.getType ();
+	if (type != COM.VT_BSTR) {
+		variant.dispose ();
+		if (pVarResult != 0) {
+			COM.MoveMemory (pVarResult, new int /*long*/[] {0}, C.PTR_SIZEOF);
+		}
+		return COM.S_OK;
+	}
+	String token = variant.getString ();
+	variant.dispose ();
+
 	variant = Variant.win32_new (dispParams.rgvarg);
 	Object key = new Integer (index);
 	BrowserFunction function = (BrowserFunction)functions.get (key);
 	Object returnValue = null;
-	if (function != null) {
+	if (function != null && token.equals (function.token)) {
 		try {
 			Object temp = convertToJava (variant);
 			if (temp instanceof Object[]) {
