@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2009 IBM Corporation and others.
+ * Copyright (c) 2000, 2011 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -1148,64 +1148,43 @@ void updateOrientation () {
 	super.updateOrientation ();
 	if (imageList != null) {
 		Point size = imageList.getImageSize ();
-		display.releaseToolImageList (imageList);
-		imageList = display.getImageListToolBar (style & SWT.RIGHT_TO_LEFT, size.x, size.y);
-		int /*long*/ hImageList = imageList.getHandle ();
-		OS.SendMessage (handle, OS.TB_SETIMAGELIST, 0, hImageList);
-	}	
-	if (hotImageList != null) {
-		Point size = hotImageList.getImageSize ();
-		display.releaseToolHotImageList (hotImageList);
-		hotImageList = display.getImageListToolBarHot (style & SWT.RIGHT_TO_LEFT, size.x, size.y);
-		int /*long*/ hHotImageList  = hotImageList.getHandle ();
-		OS.SendMessage (handle, OS.TB_SETHOTIMAGELIST, 0, hHotImageList);
-	}	
-	if (disabledImageList != null) {
-		Point size = disabledImageList.getImageSize ();
-		display.releaseToolDisabledImageList (disabledImageList);
-		disabledImageList = display.getImageListToolBarDisabled (style & SWT.RIGHT_TO_LEFT, size.x, size.y);	
-		int /*long*/ hDisImageList  = disabledImageList.getHandle ();
-		OS.SendMessage (handle, OS.TB_SETDISABLEDIMAGELIST, 0, hDisImageList);
-	}
-	if (imageList != null) {
-		for (int i = 0; i < items.length; i++) {
-			ToolItem item = items[i];
-			if (item != null) {
-				Image image = item.image;
-				if (image != null) {
-					TBBUTTONINFO info = new TBBUTTONINFO ();
-					info.cbSize = TBBUTTONINFO.sizeof;
-					info.dwMask = OS.TBIF_IMAGE;
-					boolean enabled = getEnabled () && item.getEnabled ();
-					Image disabled = item.disabledImage;
-					if (item.disabledImage == null) {
-						if (item.disabledImage2 != null) item.disabledImage2.dispose ();
-						item.disabledImage2 = null;
-						disabled = image;
-						if (!enabled) {
-							disabled = item.disabledImage2 = new Image (display, image, SWT.IMAGE_DISABLE);
-						}
-					}
-					/*
-					* Bug in Windows.  When a tool item with the style
-					* BTNS_CHECK or BTNS_CHECKGROUP is selected and then
-					* disabled, the item does not draw using the disabled
-					* image.  The fix is to assign the disabled image in
-					* all image lists.
-					*/
-					Image image2 = image, hot = item.hotImage;
-					if ((item.style & (SWT.CHECK | SWT.RADIO)) != 0) {
-						if (!enabled) image2 = hot = disabled;
-					}
-					info.iImage = imageList.add (image2);
-					disabledImageList.add (disabled);
-					hotImageList.add (hot != null ? hot : image2);
-					OS.SendMessage (handle, OS.TB_SETBUTTONINFO, item.id, info);
-				}
+		ImageList newImageList = display.getImageListToolBar (style & SWT.RIGHT_TO_LEFT, size.x, size.y);
+		ImageList newHotImageList = display.getImageListToolBarHot (style & SWT.RIGHT_TO_LEFT, size.x, size.y);
+		ImageList newDisabledImageList = display.getImageListToolBarDisabled (style & SWT.RIGHT_TO_LEFT, size.x, size.y);	
+		TBBUTTONINFO info = new TBBUTTONINFO ();
+		info.cbSize = TBBUTTONINFO.sizeof;
+		info.dwMask = OS.TBIF_IMAGE;
+		int count = (int)/*64*/OS.SendMessage (handle, OS.TB_BUTTONCOUNT, 0, 0);
+		for (int i=0; i<count; i++) {
+			ToolItem item = items [i];
+			if ((item.style & SWT.SEPARATOR) != 0) continue;
+			if (item.image == null) continue;
+			OS.SendMessage (handle, OS.TB_GETBUTTONINFO, item.id, info);
+			if (info.iImage != OS.I_IMAGENONE) {
+				Image image = imageList.get(info.iImage);
+				Image hot = hotImageList.get(info.iImage);
+				Image disabled = disabledImageList.get(info.iImage);
+				imageList.put(info.iImage, null);
+				hotImageList.put(info.iImage, null);
+				disabledImageList.put(info.iImage, null);
+				info.iImage = newImageList.add(image);
+				newHotImageList.add(hot);
+				newDisabledImageList.add(disabled);
+				OS.SendMessage (handle, OS.TB_SETBUTTONINFO, item.id, info);
 			}
 		}
+		display.releaseToolImageList (imageList);
+		display.releaseToolHotImageList (hotImageList);
+		display.releaseToolDisabledImageList (disabledImageList);
+		OS.SendMessage (handle, OS.TB_SETIMAGELIST, 0, newImageList.getHandle ());
+		OS.SendMessage (handle, OS.TB_SETHOTIMAGELIST, 0, newHotImageList.getHandle ());
+		OS.SendMessage (handle, OS.TB_SETDISABLEDIMAGELIST, 0, newDisabledImageList.getHandle ());
+		imageList = newImageList;
+		hotImageList = newHotImageList;
+		disabledImageList = newDisabledImageList;
+		OS.InvalidateRect (handle, null, true);
 	}
- }
+}
 
 int widgetStyle () {
 	int bits = super.widgetStyle () | OS.CCS_NORESIZE | OS.TBSTYLE_TOOLTIPS | OS.TBSTYLE_CUSTOMERASE;
