@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2005, 2011 IBM Corporation and others.
+ * Copyright (c) 2005, 2012 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -387,6 +387,7 @@ public class WorkbenchMessages extends NLS {
 
 
 	public static String ICategory_other;
+	public static String ICategory_general;
 
 	// ==============================================================================
 	// Wizards
@@ -404,6 +405,7 @@ public class WorkbenchMessages extends NLS {
 	public static String NewWizardsRegistryReader_otherCategory;
 	public static String NewWizardDropDown_text;
 	
+	public static String WizardHandler_menuLabel;
 	public static String WorkbenchWizard_errorMessage;
 	public static String WorkbenchWizard_errorTitle;
 	public static String WizardTransferPage_selectAll;
@@ -446,30 +448,31 @@ public class WorkbenchMessages extends NLS {
 	public static String WorkbenchPreference_RunInBackgroundToolTip;
 
 	// --- Appearance ---
-	public static String ViewsPreference_currentPresentation;
-	public static String ViewsPreference_currentPresentationFormat;
-	public static String ViewsPreference_presentationConfirm_title;
-	public static String ViewsPreference_presentationConfirm_message;
-	public static String ViewsPreference_presentationConfirm_yes;
-	public static String ViewsPreference_presentationConfirm_no;
-	public static String ViewsPreference_editors;
-	public static String ViewsPreference_editors_top;
-	public static String ViewsPreference_editors_bottom;
-	public static String ViewsPreference_views;
-	public static String ViewsPreference_views_top;
-	public static String ViewsPreference_views_bottom;
-	public static String ViewsPreference_perspectiveBar;
-	public static String ViewsPreference_perspectiveBar_topRight;
-	public static String ViewsPreference_perspectiveBar_topLeft;
-	public static String ViewsPreference_perspectiveBar_left;
-	public static String ViewsPreference_traditionalTabs;
-	public static String ViewsPreference_currentTheme;
-	public static String ViewsPreference_currentThemeDescription;
-	public static String ViewsPreference_currentThemeFormat;
+	public static String ViewsPreferencePage_Theme;
+	// public static String ViewsPreference_currentPresentation;
+	// public static String ViewsPreference_currentPresentationFormat;
+	// public static String ViewsPreference_presentationConfirm_title;
+	// public static String ViewsPreference_presentationConfirm_message;
+	// public static String ViewsPreference_presentationConfirm_yes;
+	// public static String ViewsPreference_presentationConfirm_no;
+	// public static String ViewsPreference_editors;
+	// public static String ViewsPreference_editors_top;
+	// public static String ViewsPreference_editors_bottom;
+	// public static String ViewsPreference_views;
+	// public static String ViewsPreference_views_top;
+	// public static String ViewsPreference_views_bottom;
+	// public static String ViewsPreference_perspectiveBar;
+	// public static String ViewsPreference_perspectiveBar_topRight;
+	// public static String ViewsPreference_perspectiveBar_topLeft;
+	// public static String ViewsPreference_perspectiveBar_left;
+	// public static String ViewsPreference_traditionalTabs;
+	// public static String ViewsPreference_currentTheme;
+	// public static String ViewsPreference_currentThemeDescription;
+	// public static String ViewsPreference_currentThemeFormat;
 	public static String ViewsPreference_enableAnimations;
-	public static String ViewsPreference_useColoredLabels;
-	public static String ViewsPreference_override;
-	public static String ViewsPreference_restartRequestJobName;
+	// public static String ViewsPreference_useColoredLabels;
+	// public static String ViewsPreference_override;
+	// public static String ViewsPreference_restartRequestJobName;
 	
 	// --- File Editors ---
 	public static String FileEditorPreference_fileTypes;
@@ -1007,6 +1010,7 @@ public class WorkbenchMessages extends NLS {
     public static String ContentTypes_lockedFormat;
     public static String ContentTypes_characterSetLabel;
     public static String ContentTypes_characterSetUpdateLabel;
+    public static String ContentTypes_unsupportedEncoding;
     public static String ContentTypes_fileAssociationsLabel;
     public static String ContentTypes_fileAssociationsAddLabel;
     public static String ContentTypes_fileAssociationsEditLabel;
